@@ -1,3 +1,29 @@
+/* Testcases that simulates allocation failures during hiredis-cluster API calls
+ * which verifies the handling of out of memory scenarios (OOM).
+ *
+ * These testcases overrides the default allocators by injecting own functions
+ * which can be configured to fail after a given number of successful allocations.
+ * A testcase can use a prepare function like `prepare_allocation_test()` to
+ * set the number of successful allocations that follows. The allocator will then
+ * count the number of calls before it start to return OOM failures, like
+ * malloc() returning NULL.
+ *
+ * Tests will call a hiredis-cluster API-function while iterating on a number,
+ * the number of successful allocations during the call before it hits an OOM.
+ * The result and the error code is then checked to show "Out of memory".
+ * As a last step the correct number of allocations is prepared to get a
+ * successful API-function call.
+ *
+ * Tip:
+ * When this testcase fails after code changes in the library, run the testcase
+ * in `gdb` to find which API call that failed, and in which iteration.
+ * - Go to the correct stack frame to find which API that triggered a failure.
+ * - Use the gdb command `print i` to find which iteration.
+ * - Investigate if a failure or a success is expected after the code change.
+ * - Set correct `i` in for-loop and the `prepare_allocation_test()` for the test.
+ *   Correct `i` can be hard to know, finding the correct number might require trial
+ *   and error of running with increased/decreased `i` until the edge is found.
+ */
 #include "adapters/libevent.h"
 #include "hircluster.h"
 #include "test_utils.h"
@@ -43,6 +69,10 @@ static void *hi_realloc_fail(void *ptr, size_t size) {
     return NULL;
 }
 
+/* Prepare the test fixture.
+ * Configures the allocator functions with the number of allocations
+ * that will succeed before simulating an out of memory scenario.
+ * Additionally it resets errors in the cluster context. */
 void prepare_allocation_test(redisClusterContext *cc,
                              int _successfulAllocations) {
     successfulAllocations = _successfulAllocations;
@@ -57,17 +87,21 @@ void prepare_allocation_test_async(redisClusterAsyncContext *acc,
     memset(acc->errstr, '\0', strlen(acc->errstr));
 }
 
-// Test of allocation handling
-// The testcase will trigger allocation failures during API calls.
-// It will start by triggering an allocation fault, and the next iteration
-// will start with an successfull allocation and then a failing one,
-// next iteration 2 successful and one failing allocation, and so on..
-//
-// Tip: When this testcase fails after code changes in the library,
-//      use gdb to find out which iteration that fails (print i)
-//      Update i in for-loop and the prepare_allocation_test(_, x) in
-//      the test section just after.
-void test_alloc_failure_handling() {
+/* Helper */
+redisClusterNode *getNodeByPort(redisClusterContext *cc, int port) {
+    redisClusterNodeIterator ni;
+    redisClusterInitNodeIterator(&ni, cc);
+    redisClusterNode *node;
+    while ((node = redisClusterNodeNext(&ni)) != NULL) {
+        if (node->port == port)
+            return node;
+    }
+    assert(0);
+    return NULL;
+}
+
+/* Test of allocation handling in the blocking API */
+void test_alloc_failure_handling(void) {
     int result;
     hiredisAllocFuncs ha = {
         .mallocFn = hi_malloc_fail,
@@ -188,7 +222,7 @@ void test_alloc_failure_handling() {
         redisReply *reply;
         const char *cmd = "SET key value";
 
-        cluster_node *node = redisClusterGetNodeByKey(cc, "key");
+        redisClusterNode *node = redisClusterGetNodeByKey(cc, "key");
         assert(node);
 
         // OOM failing commands
@@ -289,7 +323,7 @@ void test_alloc_failure_handling() {
         redisReply *reply;
         const char *cmd = "SET foo one";
 
-        cluster_node *node = redisClusterGetNodeByKey(cc, "foo");
+        redisClusterNode *node = redisClusterGetNodeByKey(cc, "foo");
         assert(node);
 
         // OOM failing appends
@@ -329,6 +363,133 @@ void test_alloc_failure_handling() {
         freeReplyObject(reply);
     }
 
+    // Redirects
+    {
+        /* Skip OOM testing during the prepare steps by allowing a high number of
+         * allocations. A specific number of allowed allocations will be used later
+         * in the testcase when we run commands that results in redirects. */
+        prepare_allocation_test(cc, 1000);
+
+        /* Get the source information for the migration. */
+        unsigned int slot = redisClusterGetSlotByKey("foo");
+        redisClusterNode *srcNode = redisClusterGetNodeByKey(cc, "foo");
+        int srcPort = srcNode->port;
+
+        /* Get a destination node to migrate the slot to. */
+        redisClusterNode *dstNode;
+        redisClusterNodeIterator ni;
+        redisClusterInitNodeIterator(&ni, cc);
+        while ((dstNode = redisClusterNodeNext(&ni)) != NULL) {
+            if (dstNode != srcNode)
+                break;
+        }
+        assert(dstNode && dstNode != srcNode);
+        int dstPort = dstNode->port;
+
+        redisReply *reply, *replySrcId, *replyDstId;
+
+        /* Get node id's */
+        replySrcId = redisClusterCommandToNode(cc, srcNode, "CLUSTER MYID");
+        CHECK_REPLY_TYPE(replySrcId, REDIS_REPLY_STRING);
+
+        replyDstId = redisClusterCommandToNode(cc, dstNode, "CLUSTER MYID");
+        CHECK_REPLY_TYPE(replyDstId, REDIS_REPLY_STRING);
+
+        /* Migrate slot */
+        reply = redisClusterCommandToNode(cc, srcNode,
+                                          "CLUSTER SETSLOT %d MIGRATING %s",
+                                          slot, replyDstId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+        reply = redisClusterCommandToNode(cc, dstNode,
+                                          "CLUSTER SETSLOT %d IMPORTING %s",
+                                          slot, replySrcId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+        reply = redisClusterCommandToNode(
+            cc, srcNode, "MIGRATE 127.0.0.1 %d foo 0 5000", dstPort);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+
+        /* Test ASK reply handling with OOM */
+        for (int i = 0; i < 50; ++i) {
+            prepare_allocation_test(cc, i);
+            reply = redisClusterCommand(cc, "GET foo");
+            assert(reply == NULL);
+            ASSERT_STR_EQ(cc->errstr, "Out of memory");
+        }
+
+        /* Test ASK reply handling without OOM */
+        prepare_allocation_test(cc, 50);
+        reply = redisClusterCommand(cc, "GET foo");
+        CHECK_REPLY_STR(cc, reply, "one");
+        freeReplyObject(reply);
+
+        /* Finalize the migration. Skip OOM testing during these steps by
+         * allowing a high number of allocations. */
+        prepare_allocation_test(cc, 1000);
+        /* Fetch the nodes again, in case the slotmap has been reloaded. */
+        srcNode = redisClusterGetNodeByKey(cc, "foo");
+        dstNode = getNodeByPort(cc, dstPort);
+        reply = redisClusterCommandToNode(
+            cc, srcNode, "CLUSTER SETSLOT %d NODE %s", slot, replyDstId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+        reply = redisClusterCommandToNode(
+            cc, dstNode, "CLUSTER SETSLOT %d NODE %s", slot, replyDstId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+
+        /* Test MOVED reply handling with OOM */
+        for (int i = 0; i < 34; ++i) {
+            prepare_allocation_test(cc, i);
+            reply = redisClusterCommand(cc, "GET foo");
+            assert(reply == NULL);
+            ASSERT_STR_EQ(cc->errstr, "Out of memory");
+        }
+
+        /* Test MOVED reply handling without OOM */
+        prepare_allocation_test(cc, 34);
+        reply = redisClusterCommand(cc, "GET foo");
+        CHECK_REPLY_STR(cc, reply, "one");
+        freeReplyObject(reply);
+
+        /* MOVED triggers a slotmap update which currently replaces all cluster_node
+         * objects. We can get the new objects by searching for its server ports.
+         * This enables us to migrate the slot back to the original node. */
+        srcNode = getNodeByPort(cc, srcPort);
+        dstNode = getNodeByPort(cc, dstPort);
+
+        /* Migrate back slot, required by the next testcase. Skip OOM testing
+         * during these final steps by allowing a high number of allocations. */
+        prepare_allocation_test(cc, 1000);
+        reply = redisClusterCommandToNode(cc, dstNode,
+                                          "CLUSTER SETSLOT %d MIGRATING %s",
+                                          slot, replySrcId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+        reply = redisClusterCommandToNode(cc, srcNode,
+                                          "CLUSTER SETSLOT %d IMPORTING %s",
+                                          slot, replyDstId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+        reply = redisClusterCommandToNode(
+            cc, dstNode, "MIGRATE 127.0.0.1 %d foo 0 5000", srcPort);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+        reply = redisClusterCommandToNode(
+            cc, dstNode, "CLUSTER SETSLOT %d NODE %s", slot, replySrcId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+        reply = redisClusterCommandToNode(
+            cc, srcNode, "CLUSTER SETSLOT %d NODE %s", slot, replySrcId->str);
+        CHECK_REPLY_OK(cc, reply);
+        freeReplyObject(reply);
+
+        freeReplyObject(replySrcId);
+        freeReplyObject(replyDstId);
+    }
+
     redisClusterFree(cc);
     hiredisResetAllocators();
 }
@@ -363,11 +524,7 @@ void commandCallback(redisClusterAsyncContext *cc, void *r, void *privdata) {
 }
 
 // Test of allocation handling in async context
-// The testcase will trigger allocation failures during API calls.
-// It will start by triggering an allocation fault, and the next iteration
-// will start with an successfull allocation and then a failing one,
-// next iteration 2 successful and one failing allocation, and so on..
-void test_alloc_failure_handling_async() {
+void test_alloc_failure_handling_async(void) {
     int result;
     hiredisAllocFuncs ha = {
         .mallocFn = hi_malloc_fail,
@@ -484,7 +641,7 @@ void test_alloc_failure_handling_async() {
     event_base_free(base);
 }
 
-int main() {
+int main(void) {
 
     test_alloc_failure_handling();
     test_alloc_failure_handling_async();
