@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Carp 'croak';
 
-our $VERSION = "0.093";
+our $VERSION = "0.094";
 
 use constant {
     DEFAULT_COMMAND_TIMEOUT => 1.0,
@@ -55,6 +55,13 @@ sub new {
     return $self;
 }
 
+sub run_event_loop {
+    my $self = shift;
+    my $result = $self->__run_event_loop();
+    return undef if $result == -1;
+    return $result;
+}
+
 sub wait_one_response {
     my $self = shift;
     my $result = $self->__wait_one_response();
@@ -185,25 +192,22 @@ This client start to connect using RESP2 and currently it has no option to upgra
 
 Simple microbenchmark comparing PP and XS.
 The benchmark script used can be found under examples directory.
+Each operation was executed 100,000 times, and the execution time was measured in milliseconds.
 
-    Redis::Cluster::Fast is 0.084
-    Redis::ClusterRider is 0.26
-    ### mset ###
-                            Rate  Redis::ClusterRider Redis::Cluster::Fast
-    Redis::ClusterRider  13245/s                   --                 -34%
-    Redis::Cluster::Fast 20080/s                  52%                   --
-    ### mget ###
-                            Rate  Redis::ClusterRider Redis::Cluster::Fast
-    Redis::ClusterRider  14641/s                   --                 -40%
-    Redis::Cluster::Fast 24510/s                  67%                   --
-    ### incr ###
-                            Rate  Redis::ClusterRider Redis::Cluster::Fast
-    Redis::ClusterRider  18367/s                   --                 -44%
-    Redis::Cluster::Fast 32879/s                  79%                   --
-    ### new and ping ###
-                           Rate  Redis::ClusterRider Redis::Cluster::Fast
-    Redis::ClusterRider   146/s                   --                 -96%
-    Redis::Cluster::Fast 3941/s                2598%                   --
+    +--------------------------------+-------+-------+-------+-------+-------+
+    | Operation                      | P50   | P80   | P95   | P99   | P100  |
+    +--------------------------------+-------+-------+-------+-------+-------+
+    | get_pp                         | 0.028 | 0.032 | 0.036 | 0.050 | 0.880 |
+    | get_xs                         | 0.020 | 0.023 | 0.025 | 0.044 | 0.881 |
+    | get_xs_pipeline                | 0.014 | 0.015 | 0.018 | 0.021 | 0.472 |
+    | get_xs_pipeline_batched_100    | 0.003 | 0.003 | 0.004 | 0.074 | 0.323 |
+    | set_pp                         | 0.028 | 0.032 | 0.037 | 0.051 | 2.014 |
+    | set_xs                         | 0.021 | 0.024 | 0.027 | 0.047 | 0.729 |
+    | set_xs_pipeline                | 0.014 | 0.016 | 0.018 | 0.021 | 0.393 |
+    | set_xs_pipeline_batched_100    | 0.003 | 0.004 | 0.005 | 0.073 | 0.379 |
+    +--------------------------------+-------+-------+-------+-------+-------+
+
+c.f. https://github.com/plainbanana/Redis-Cluster-Fast-Benchmarks
 
 =head1 METHODS
 
@@ -278,14 +282,14 @@ To run a Redis command in pipeline with arguments and a callback.
 The command can also be expressed by concatenating the subcommands with underscores.
 
 Commands issued to the same node are sent and received in pipeline mode.
-In pipeline mode, commands are not sent to Redis until C<wait_one_response> or C<wait_all_responses> is issued.
+In pipeline mode, commands are not sent to Redis until C<run_event_loop>, C<wait_one_response> or C<wait_all_responses> is issued.
 
 The callback is executed with two arguments.
 The first is the result of the command, and the second is the error message.
 C<$result> will be a scalar value or an array reference, and C<$error> will be an undefined value if no errors occur.
 Also, C<$error> may contain an error returned from Redis or an error that occurred on the client (e.g. Timeout).
 
-You cannot call any client methods inside the callback.
+You cannot call any client methods or exceptions inside the callback.
 
 After issuing a command in pipeline mode,
 do not execute fork() without issuing C<disconnect> if all callbacks are not executed completely.
@@ -295,6 +299,64 @@ do not execute fork() without issuing C<disconnect> if all callbacks are not exe
         # some operations...
     });
 
+=head2 run_event_loop()
+
+This method allows you to issue commands without waiting for their responses.
+You can then perform a blocking wait for those responses later, if needed.
+
+Executes one iteration of the event loop to process any pending commands that have not yet been sent
+and any incoming responses from Redis.
+
+If there are events that can be triggered immediately, they will all be processed.
+In other words, if there are unsent commands, they will be pipelined and sent,
+and if there are already-received responses, their corresponding callbacks will be executed.
+
+If there are no events that can be triggered immediately: there are neither unsent commands nor any Redis responses available to read,
+but unprocessed callbacks remain, then this method will block for up to C<command_timeout> while waiting for a response from Redis.
+When a timeout occurs, an error will be propagated to the corresponding callback(s).
+
+The return value can be either 1 for success (e.g., commands sent or responses read),
+0 for no callbacks remained, or undef for other errors.
+
+=head3 Notes
+
+=over 4
+
+=item *
+
+Be aware that the timeout check will only be triggered when there are neither unsent commands nor Redis responses available to read.
+If a timeout occurs, all remaining commands on that node will time out as well.
+
+=item *
+
+Internally, this method calls C<event_base_loop(..., EVLOOP_ONCE)>, which
+performs a single iteration of the event loop. A command will not be fully processed in a single call.
+
+=item *
+
+If you need to process multiple commands or wait for all responses, call
+this method repeatedly or use C<wait_all_responses>.
+
+=item *
+
+For a simpler, synchronous-like usage where you need at least one response,
+refer to C<wait_one_response>. If you only need to block until all
+pending commands are processed, see C<wait_all_responses>.
+
+=back
+
+=head3 Example
+
+  # Queue multiple commands in pipeline mode
+  $redis->set('key1', 'value1', sub {});
+  $redis->get('key2', sub {});
+
+  # Send commands to Redis without waiting for responses
+  $redis->run_event_loop();
+
+  # Possibly wait for responses
+  $redis->run_event_loop();
+
 =head2 wait_one_response()
 
 If there are any unexcuted callbacks, it will block until at least one is executed.
